"""
Job search source implementations.

Each module in this package provides a specific job search source adapter.
"""

# Import individual sources here for easy access
# They will be registered automatically when imported

__all__ = []
